@@ -115,7 +115,7 @@
 
     (long-class-specifier
      (ident string-comment composition "end" ident
-	    ($$ (check-ids $1 $5) (if (pair? $2) (list $1 $2 $3) (list $1 $3))))
+	    ($$ (check-ids $1 $5) (list $1 $3)))
      ("extends" ident class-modification string-comment composition "end" ident
       ($$ (check-ids $1 $5) (list '(@ extends . "yes") $2 $3 $4 $5)))
      ("extends" ident string-comment composition "end" ident
@@ -123,36 +123,42 @@
      )
 
     (short-class-specifier
-     (ident "=" base-prefix type-specifier array-subscripts
-	    class-modification comment
-	    ($$ (list $1 `(is ,$3 ,$4 ,$5 ,$6 ,$7))))
+     (ident "=" base-prefix type-specifier array-subscripts class-modification
+	    comment
+	    ($$ (if (pair? $3)
+		    `(short-class-spec ,$1 ,$3 ,$4 ,$5 ,$6)
+		    `(short-class-spec ,$1 ,$4 ,$5 ,$6))))
      (ident "=" base-prefix type-specifier array-subscripts comment
-	    )
-     
-     (ident "=" base-prefix type-specifier class-modification comment)
-     (ident "=" base-prefix type-specifier comment)
-     (ident "=" "enumeration" "(" enum-list ")" comment)
-     (ident "=" "enumeration" "(" ":" ")" comment)
+	    ($$ (if (pair? $3)
+		    `(short-class-spec ,$1 ,$3 ,$4 ,$5)
+		    `(short-class-spec ,$1 ,$4 ,$5))))
+     (ident "=" base-prefix type-specifier class-modification comment
+	    ($$ (if (pair? $3)
+		    `(short-class-spec ,$1 ,$3 ,$4)
+		    `(short-class-spec ,$1 ,$4))))
+     (ident "=" "enumeration" "(" enum-list ")" comment
+	    ($$ `(short-class-enum-spec ,$1 ,$5)))
+     (ident "=" "enumeration" "(" ":" ")" comment
+	    ($$ `(short-class-enum-spec ,$1 ,$5)))
      )
 
     (der-class-specifier
      (ident "=" "der" "(" type-specifier "," der-class-specifier-1 ")" comment
-	    ($$ `(der-class-specifier ... (tl->list $7))))
-     )
+	    ($$ `(der-class-specifier ,$1 ,$5 ,$7))))
     (der-class-specifier-1 ;; or ident-colon-list
+      (ident-list-1 ($$ (tl->list $1))))
+    (ident-list-1
      (ident ($$ (make-tl 'ident-list $1)))
-     (der-class-specifier-1 ";" ident ($$ (tl-append $1 $3)))
-     )
+     (ident-list-1 ";" ident ($$ (tl-append $1 $3))))
 
-    (base-prefix ("input") ("output"))
+    (base-prefix ($empty) ("input") ("output"))
 
     (enum-list
      (enumeration-literal ($$ (make-tl 'enum-list $1)))
      (enum-list "," enumeration-literal ($$ (tl-append $1 $3))))
 
     (enumeration-literal
-     (ident comment)
-     )
+     (ident comment ($$ $1)))
 
     ;; ===================== update to v3.4 stopped here ======================
 
@@ -201,10 +207,14 @@
     (language-specification (string))
 
     (external-function-call
-     (component-reference "=" ident "(" expression-list ")")
-     (component-reference "=" ident "(" ")")
-     (ident "(" expression-list ")")
-     (ident "(" ")")
+     (component-reference "=" ident "(" expression-list ")"
+			  ($$ `(ext-fctn-call ,$1 ,$3 ,$5)))
+     (component-reference "=" ident "(" ")"
+			  ($$ `(ext-fctn-call ,$1 ,$3 '(expr-list))))
+     (ident "(" expression-list ")"
+			  ($$ `(ext-fctn-call ,$1 ,$3)))
+     (ident "(" ")"
+			  ($$ `(ext-fctn-call ,$1 '(expr-list))))
      )
 
     (element-list
@@ -223,10 +233,11 @@
      (element-1)
       )
     (element-1
-     (class-definition)
-     (component-clause)
-     ("replaceable" element-2 constraining-clause comment)
-     ("replaceable" element-2)
+     (element-2)
+     ("replaceable" element-2 constraining-clause comment
+      )
+     ("replaceable" element-2
+      )
      )
     (element-2
      (class-definition)
@@ -252,16 +263,20 @@
 
     ;; B.2.3 Extends
     (extends-clause
-     ("extends" name class-modification annotation)
-     ("extends" name class-modification)
-     ("extends" name annotation)
-     ("extends" name)
-     )
+     ("extends" name class-modification annotation
+      ($$ `(extends ,$2 ,$3)))
+     ("extends" name class-modification
+      ($$ `(extends ,$2 ,$3)))
+     ("extends" name annotation
+      ($$ `(extends ,$2)))
+     ("extends" name
+      ($$ `(extends ,$2))))
 
     (constraining-clause
-     ("constrainedby" name class-modification)
-     ("constrainedby" name)
-     )
+     ("constrainedby" name class-modification
+      ($$ `(constrained-by ,$1 ,$2)))
+     ("constrainedby" name
+      ($$ `(constrained-by ,$1))))
 
     ;; B.2.4 Component Clause
     (component-clause
@@ -293,72 +308,68 @@
     (type-prefix-2 ("discrete") ("parameter") ("constant"))
     (type-prefix-3 ("input") ("output"))
 
-    (type-specifier (name))
+    (type-specifier (name ($$ `(type-spec ,$1))))
 
     (component-list
-     (component-declaration)
-     (component-list "," component-declaration)
-     )
+     (component-list-1 ($$ (tl->list $1))))
+    (component-list-1
+     (component-declaration ($$ (make-tl 'comp-list $1)))
+     (component-list-1 "," component-declaration ($$ (tl-append $1 $3))))
 
     (component-declaration
-     (declaration condition-attribute comment)
-     (declaration comment)
-     )
+     (declaration condition-attribute comment ($$ `(comp-decl ,$1 ,$2)))
+     (declaration comment ($$ `(comp-decl ,$1))))
 
     (condition-attribute
-     ("if" expression)
-     )
+     ("if" expression ($$ `(if ,$2))))
 
     (declaration
-     (ident ($? array-subscripts) ($? modification))
-     )
+     (ident ($? array-subscripts) ($? modification)
+	    ($$ (make-sx 'decl #f $1 $2 $3))))
 
     ;; B.2.5 Modification
     (modification
-     (class-modification "=" expression)
-     (class-modification)
-     ("=" expression)
-     (":=" expression)
-     )
+     (class-modification "=" expression ($$ `(class-mod ,$1 ,$3)))
+     (class-modification ($$ `(class-mod ,$1)))
+     ("=" expression ($$ `(eqv-mod ,$2)))
+     (":=" expression ($$ `(def-mod ,$2))))
 
     (class-modification
-     ("(" argument-list ")")
-     ("(" ")")
-     )
+     ("(" argument-list ")" ($$ $2))
+     ("(" ")" ($$ '(arg-list))))
 
     (argument-list
-     (argument)
-     (argument-list "," argument)
-     )
+     (arg-list-1 ($$ (tl->list $1))))
+    (arg-list-1
+     (argument ($$ (make-tl 'arg-list $1)))
+     (arg-list-1 "," argument ($$ (tl-append $1 $3))))
 
     (argument (element-modification-or-replaceable) (element-redeclaration))
 
     (element-modification-or-replaceable
-     ("each" "final" elt-mod-or-repl-1)
-     ("each" elt-mod-or-repl-1)
-     ("final" elt-mod-or-repl-1)
-     (elt-mod-or-repl-1)
-     )
+     ("each" "final" elt-mod-or-repl-1 ($$ $3))
+     ("each" elt-mod-or-repl-1 ($$ $2))
+     ("final" elt-mod-or-repl-1 ($$ $2))
+     (elt-mod-or-repl-1 ($$ $1)))
     (elt-mod-or-repl-1 (element-modification) (element-replaceable))
 
-    (element-modification (name ($? modification) string-comment))
+    (element-modification
+     (name ($? modification) string-comment
+	   ($$ (if (pair? $2) `(elt-mod ,$1 ,$2) `(elt-mod ,$1)))))
 
     ;; This looks wierd in the 3.3r1 spec. Like maybe typo.
     (element-redeclaration
-     ("redeclare" ($? "each") ($? "final") elt-redecl-1)
-     )
+     ("redeclare" ($? "each") ($? "final") elt-redecl-1 ($$ $4)))
     (elt-redecl-1
      (short-class-definition)
      (component-clause1)
-     (element-replaceable)
-     )
+     (element-replaceable))
 
     (element-replaceable
-     ("replaceable"
-      short-class-definition component-clause1 constraining-clause)
-     ("replaceable"
-      short-class-definition component-clause1)
-     )
+     ("replaceable" short-class-definition component-clause1 constraining-clause
+      ($$ `(elt-repl $2 $3 $4)))
+     ("replaceable" short-class-definition component-clause1
+      ($$ `(elt-repl $2 $3))))
 
     (component-clause1
      (type-prefix type-specifier declaration comment
@@ -372,26 +383,24 @@
     ;; B.2.6 Equations
     (equation-section
      ("initial" "equation" equation-list
-      ($$ `(init-eqn-section ,$3)))
+      ($$ `(init-eqn-section . ,(cdr $3))))
      ("equation" equation-list
-      ($$ `(eqn-section ,$2)))
+      ($$ `(eqn-section . ,(cdr $2))))
      ("initial" "equation"
-      ($$ `(init-eqn-section (eqn-list))))
+      ($$ `(init-eqn-section)))
      ("equation"
-      ($$ `(eqn-section (eqn-list))))
-     )
+      ($$ `(eqn-section))))
 
     ;; I think they messed this up tool
     (algorithm-section
      ("initial" "algorithm" statement-list
-      ($$ `(init-alg-section ,$3)))
+      ($$ `(init-alg-section . ,(cdr $3))))
      ("algorithm" statement-list
-      ($$ `(alg-section ,$2)))
+      ($$ `(alg-section . ,(cdr $2))))
      ("initial" "algorithm"
-      ($$ `(init-alg-section (stmt-list))))
+      ($$ `(init-alg-section)))
      ("algorithm"
-      ($$ `(alg-section (stmt-list))))
-     )
+      ($$ `(alg-section))))
 
     ;; my addition:
     (equation-list
@@ -401,7 +410,8 @@
      (equation-list-1 equation ";" ($$ (tl-append $1 $2)))
      )
 
-    (equation (equation-1 comment ($$ (append $1 (list $2)))))
+    (equation
+     (equation-1 comment ($$ $1)))
     (equation-1
      (simple-expression "=" expression ($$ `(equate ,$1 ,$3)))
      (if-equation)
@@ -415,7 +425,8 @@
      (statement ";" ($$ (make-tl 'stmt-list $1)))
      (statement-list statement ";" ($$ (tl-append $1 $2))))
 
-    (statement (statement-1 comment ($$ (append $1 (list $2)))))
+    (statement
+     (statement-1 comment ($$ $1)))
     (statement-1
      (component-reference ":=" expression ($$ `(assign ,$1 ,$3)))
      (component-reference function-call-args ($$ `(call ,$1 ,$2)))
@@ -586,35 +597,39 @@
 
     (term
      (factor)
-     (term mul-op factor)
-     )
+     (term mul-op factor ($$ (list $2 $1 $3))))
     (mul-op ("*" ($$ 'mul)) ("/" ($$ 'div))
 	    (".*" ($$ 'dot-mul)) ("./" ($$ 'dot-div)))
 
     (factor
-     (primary)
-     (factor "^" primary ($$ `(pow ,$1 ,$2)))
-     (factor ".^" primary ($$ `(dot-pow ,$1 ,$2)))
+     (unary-expr)
+     (factor "^" unary-expr ($$ `(pow ,$1 ,$2)))
+     (factor ".^" unary-expr ($$ `(dot-pow ,$1 ,$2)))
      )
 
+    (unary-expr
+     (primary)
+     ("+" primary ($$ `(pos ,$2)))
+     ("-" primary ($$ `(neg ,$2))))
+
     (primary
      (unsigned-number ($$ `(p-expr ,$1)))
      (string ($$ `(p-expr ,$1)))
      ("false" ($$ `(p-expr '(false))))
      ("true" ($$ `(p-expr '(true))))
-     (name function-call-args ($$ `(fctn-call ,$1 ,(tl->list $2))))
+     (name function-call-args ($$ `(fctn-call ,$1 ,$2)))
      ("der" function-call-args ($$ `(der ,$2)))
      ;;("initial" function-call-args)  ; 4 srconf, OK to shift?
      
      ;; from component reference:
-     (name)
+     (name ($$ `(p-expr ,$1)))
      (name array-subscripts ($$ `(array-elt ,$1 ,$2)))
      
-     ("(" output-expression-list ")")
+     ("(" output-expression-list ")" ($$ `(p-expr ,$2)))
      ("[" expression-list-list  "]"
       ($$ `(matrix ,(map (lambda (row) (cons 'row (cdr row)))
 			 (cdr (tl->list $2))))))
-     ("{" function-arguments "}")
+     ("{" function-arguments "}" ($$ `(??? ,$2)))
      ;;("end")			     ; 2 srconf, and WTF is this for?
      )
     (expression-list-list
@@ -663,11 +678,14 @@
     ;;(function-argument-1 ("," function-arguments) ("for" for-indices))
     ;;(named-arguments (named-argument) (named-arguments "," named-argument))
 
-    (named-argument (ident "=" function-argument))
+    (named-argument
+     (ident "=" function-argument ($$ `(named-arg ,$1 ,$3))))
 
     (function-argument
-     ("function" name "(" named-arguments ")")
-     ("function" name "(" ")")
+     ("function" name "(" named-arguments ")"
+      ($$ `(fctn-arg ,$2 ,$4)))
+     ("function" name "(" ")"
+      ($$ `(fctn-arg ,$2)))
      (expression)
      )
 
@@ -678,9 +696,10 @@
      )
 
     (expression-list
-     (expression)
-     (expression-list "," expression)
-     )
+     (expression-list-1 ($$ (tl->list $1))))
+    (expression-list-1
+     (expression ($$ (make-tl 'expr-list $1)))
+     (expression-list-1 "," expression ($$ (tl-append $1 $3))))
 
     (array-subscripts
      ("[" array-subscript-list "]" ($$ (tl->list $2))))
@@ -697,8 +716,7 @@
     (comment
      (string-comment annotation
 		     ($$ (if (pair? $1) `(comment ,$1 ,$2) `(comment ,$2))))
-     (string-comment ($$ (if (pair? $1) `(comment ,$1) '(comment))))
-     )
+     (string-comment ($$ (if (pair? $1) `(comment ,$1) '()))))
 
     (string-comment
      ($empty)
@@ -709,7 +727,7 @@
      (string ($$ (make-tl 'string-comment $1)))
      (string-cat-1 "+" string ($$ (tl-append $1 $3))))
     
-    (opt-annotation () (annotation ";"))
+    (opt-annotation ($empty) (annotation ";"))
     (annotation
      ("annotation" class-modification)
      )
