@@ -85,7 +85,7 @@
   #:use-module (ice-9 rdelim)
   #:use-module (ice-9 regex)
   #:re-export (*nyacc-version*)
-  #:version (0 83 0)
+  #:version (0 86 6)
   )
 
 (define (fhoo) #t)			; debugging
@@ -855,7 +855,7 @@
 	 (mdecl (udecl->mdecl udecl1)))
     (mtail->bs-desc (cdr mdecl))))
 
-(define (int->abs-ident ix)
+(define (int->name ix)
   (simple-format #f "arg-~A" ix))
 
 (define (gen-decl-params params)
@@ -870,7 +870,7 @@
       ;;(sferr "\nP: ~S\n" (car params))
       (let* ((udecl1 (expand-typerefs (car params) (*udict*) ffi-defined))
 	     (udecl1 (udecl-rem-type-qual udecl1))
-	     (mdecl (udecl->mdecl udecl1 #:abs-ident (int->abs-ident ix))))
+	     (mdecl (udecl->mdecl udecl1 #:add-name (int->name ix))))
 	;;(sferr "  ~S\n" udecl1)
 	(cons (mtail->ffi-desc (cdr mdecl))
 	      (iter (1+ ix) (cdr params))))))))
@@ -885,7 +885,7 @@
      (else
       (let* ((udecl1 (expand-typerefs (car params) (*udict*) ffi-defined))
 	     (udecl1 (udecl-rem-type-qual udecl1))
-	     (mdecl (udecl->mdecl udecl1 #:abs-ident (int->abs-ident ix))))
+	     (mdecl (udecl->mdecl udecl1 #:add-name (int->name ix))))
 	(cons (mtail->bs-desc (cdr mdecl))
 	      (iter (1+ ix) (cdr params))))))))
 
@@ -1166,9 +1166,7 @@
 
 ;; assume unit-declarator
 ;; TODO (ptr-declr (pointer (type-qual-list (type-qual "const"))))
-;; todo:
-;; 1) remove comments
-;; 2) keep attributes! Used for forward decl's
+;; See also stripdown-specl and stripdown-declr in @file{munge.scm}.
 (define (cleanup-udecl specl declr)
   (let* ((fctn? (pair? ((sxpath '(// ftn-declr)) declr)))
 	 (specl (remove (lambda (node)
@@ -1176,14 +1174,12 @@
 			      (equal? node '(stor-spec (register)))
 			      (equal? node '(stor-spec (static)))
 			      (and (pair? node)
-				   (equal? (car node) 'type-qual))
-			      ))
+				   (equal? (car node) 'type-qual))))
 			specl))
 	 (specl (if fctn?
 		    (remove (lambda (node)
 			      (equal? node '(stor-spec (extern)))) specl)
-		    specl))
-	 )
+		    specl)))
     (values specl declr)))
 
 ;; @deffn {Procecure} back-ref-extend! decl typename
@@ -1229,7 +1225,7 @@
   (*wrapped* wrapped)
   (*defined* defined)
 
-  (let*-values (((tag attr specl declr tail) (split-adecl udecl))
+  (let*-values (((tag attr specl declr) (split-udecl udecl))
 		((specl declr) (cleanup-udecl specl declr))
 		((clean-udecl) (values (sx-list tag #f specl declr)))
 		)
