@@ -1064,16 +1064,15 @@
     43) ("double" . 44) ("float" . 45) ("char" . 46) ("unsigned" . 47) 
     ("long" . 48) ("signed" . 49) ("int" . 50) (imp . 51) ("short" . 52) 
     ("_Bool" . 53) ("void" . 54) ("typedef" . 55) ("static" . 56) ("register" 
-    . 57) ("extern" . 58) ("auto" . 59) (reduce-on-attr-spec . 60) (
-    shift-on-attr-spec . 61) (";" . 62) ("|=" . 63) ("^=" . 64) ("&=" . 65) 
-    (">>=" . 66) ("<<=" . 67) ("%=" . 68) ("/=" . 69) ("*=" . 70) ("-=" . 71) 
-    ("+=" . 72) ("=" . 73) (":" . 74) ("?" . 75) ("||" . 76) ("&&" . 77) 
-    ("|" . 78) ("^" . 79) ("!=" . 80) ("==" . 81) (">=" . 82) ("<=" . 83) 
-    (">" . 84) ("<" . 85) (">>" . 86) ("<<" . 87) ("%" . 88) ("/" . 89) 
-    ("!" . 90) ("~" . 91) ("-" . 92) ("+" . 93) ("*" . 94) ("&" . 95) (
-    "sizeof" . 96) ("," . 97) ("--" . 98) ("++" . 99) ("->" . 100) ("." . 101)
-    ("]" . 102) ("[" . 103) ("}" . 104) ("{" . 105) (")" . 106) ("(" . 107) 
-    ($error . 2) ($end . 109)))
+    . 57) ("extern" . 58) ("auto" . 59) (reduce-on-attr . 60) (shift-on-attr 
+    . 61) (";" . 62) ("|=" . 63) ("^=" . 64) ("&=" . 65) (">>=" . 66) ("<<=" 
+    . 67) ("%=" . 68) ("/=" . 69) ("*=" . 70) ("-=" . 71) ("+=" . 72) ("=" . 
+    73) (":" . 74) ("?" . 75) ("||" . 76) ("&&" . 77) ("|" . 78) ("^" . 79) 
+    ("!=" . 80) ("==" . 81) (">=" . 82) ("<=" . 83) (">" . 84) ("<" . 85) 
+    (">>" . 86) ("<<" . 87) ("%" . 88) ("/" . 89) ("!" . 90) ("~" . 91) 
+    ("-" . 92) ("+" . 93) ("*" . 94) ("&" . 95) ("sizeof" . 96) ("," . 97) 
+    ("--" . 98) ("++" . 99) ("->" . 100) ("." . 101) ("]" . 102) ("[" . 103) 
+    ("}" . 104) ("{" . 105) (")" . 106) ("(" . 107) ($error . 2) ($end . 109)))
 
 (define c99-tables
   (list
